@@ -15,6 +15,8 @@ Config::Config( Logger *_logger, QObject *parent )
     : QObject( parent ),
     logger( _logger )
 {
+    connect( this, SIGNAL(updateWriteLogFilesSetting(bool)), logger, SLOT(updateWriteSetting(bool)) );
+
     pPluginLoader = new PluginLoader( logger, this );
     pTagEngine = new TagEngine();
     pConversionOptionsManager = new ConversionOptionsManager( pPluginLoader );
@@ -53,8 +55,10 @@ void Config::load()
     data.general.copyStructureOutputDirectory = group.readEntry( "copyStructureOutputDirectory", QDir::homePath() + "/soundKonverter" );
     data.general.lastMetaDataOutputDirectoryPaths = group.readEntry( "lastMetaDataOutputDirectoryPaths", QStringList() );
     data.general.lastNormalOutputDirectoryPaths = group.readEntry( "lastNormalOutputDirectoryPaths", QStringList() );
-//     data.general.waitForAlbumGain = group.readEntry( "waitForAlbumGain", false );
+//     data.general.waitForAlbumGain = group.readEntry( "waitForAlbumGain", true );
+    data.general.waitForAlbumGain = false;
     data.general.useVFATNames = group.readEntry( "useVFATNames", false );
+    data.general.writeLogFiles = group.readEntry( "writeLogFiles", false );
     data.general.conflictHandling = (Config::Data::General::ConflictHandling)group.readEntry( "conflictHandling", 0 );
 //     data.general.priority = group.readEntry( "priority", 10 );
     data.general.numFiles = group.readEntry( "numFiles", 0 );
@@ -94,6 +98,9 @@ void Config::load()
     }
     data.advanced.maxSizeForSharedMemoryTempFiles = group.readEntry( "maxSizeForSharedMemoryTempFiles", data.advanced.sharedMemorySize / 2 );
 
+    group = conf->group( "CoverArt" );
+    data.coverArt.writeCovers = group.readEntry( "writeCovers", 1 );
+
     group = conf->group( "Backends" );
     data.backends.rippers = group.readEntry( "rippers", QStringList() );
     formats = group.readEntry( "formats", QStringList() );
@@ -438,7 +445,9 @@ void Config::save()
     group.writeEntry( "copyStructureOutputDirectory", data.general.copyStructureOutputDirectory );
     group.writeEntry( "lastMetaDataOutputDirectoryPaths", data.general.lastMetaDataOutputDirectoryPaths );
     group.writeEntry( "lastNormalOutputDirectoryPaths", data.general.lastNormalOutputDirectoryPaths );
+//     group.writeEntry( "waitForAlbumGain", data.general.waitForAlbumGain );
     group.writeEntry( "useVFATNames", data.general.useVFATNames );
+    group.writeEntry( "writeLogFiles", data.general.writeLogFiles );
     group.writeEntry( "conflictHandling", (int)data.general.conflictHandling );
 //     group.writeEntry( "priority", data.general.priority );
     group.writeEntry( "numFiles", data.general.numFiles );
@@ -455,6 +464,9 @@ void Config::save()
     group.writeEntry( "useSharedMemoryForTempFiles", data.advanced.useSharedMemoryForTempFiles );
     group.writeEntry( "maxSizeForSharedMemoryTempFiles", data.advanced.maxSizeForSharedMemoryTempFiles );
 
+    group = conf->group( "CoverArt" );
+    group.writeEntry( "writeCovers", data.coverArt.writeCovers );
+
     group = conf->group( "Backends" );
     group.writeEntry( "rippers", data.backends.rippers );
     for( int i=0; i<data.backends.codecs.count(); i++ )
@@ -492,6 +504,8 @@ void Config::save()
     }
 
     writeServiceMenu();
+
+    emit updateWriteLogFilesSetting( data.general.writeLogFiles );
 }
 
 void Config::writeServiceMenu()
